/*!
 * @file resource.h
 * @brief Definition of a Resource.
*/

#ifndef _RESOURCE_H
#define _RESOURCE_H

#include "base_object.h"
#include "multi_type.h"
#include <string>
#include <vector>
#include <set>

//! A Resource is an Object, that can be loaded from Disk
/**
 *
 */
class Resource : virtual public BaseObject
{
public:
  class Priority
  {
  public:
    void setPriorityName(unsigned int level, const std::string& name)
    {
      if (_priorityName.size() <= level)
        _priorityName.resize(level+1);
      _priorityName[level] = name;
    }
  private:
    std::vector<std::string>     _priorityName;
  };


  class Type
  {
  public:
    Type(const ClassID& classID) : _classID(classID) { };

    void addExtension(const std::string& extension);

  private:
    const ClassID&                _classID;
    std::vector<std::string>      _resourcePaths;
    std::vector<std::string>      _resourceSubPaths;
    std::vector<std::string>      _fileExtensions;

  };


public:
  Resource(const std::string& fileName);
  virtual ~Resource();

  virtual bool load(std::string& fileName, const MultiType& param1, const MultiType& param2);
  virtual bool reload();
  virtual bool unload();


private:

  static std::vector<std::vector<std::string> >     _resourcePaths;

  static std::vector<std::string>                   _resourceClassNames;

  //! GLOBALS
  static std::string                                _mainGlobalPath;
  static std::vector<std::string>                   _globalPaths;

  std::string       fileName;

  unsigned int      referenceCount;    //!< How many times this Resource has been loaded.
  /// TODO REMOVE THIS:   ResourceType      type;              //!< ResourceType of this Resource.
  Resource::Priority  prio;              //!< The Priority of this resource. (can only be increased, so noone else will delete this)

  MultiType         param[3];          //!< The Parameters given to this Resource.
};

#endif /* _RESOURCE_H */
